library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity opb_vidprocessor is  -- USER --
  generic
  (
    C_OPB_AWIDTH       : integer          := 32;
    C_OPB_DWIDTH       : integer          := 32;
    C_BASEADDR         : std_logic_vector := X"FEFF0300";
    C_HIGHADDR         : std_logic_vector := X"FEFF03FF");
  
  port
  (
    --Required OPB bus ports, do not add to or delete
    OPB_ABus     : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE       : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk      : in  std_logic;
    OPB_DBus     : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW      : in  std_logic;
    OPB_Rst      : in  std_logic;
    OPB_select   : in  std_logic;
    OPB_seqAddr  : in  std_logic;
    VIDPROC_DBus     : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    VIDPROC_errAck   : out std_logic;
    VIDPROC_retry    : out std_logic;
    VIDPROC_toutSup  : out std_logic;
    VIDPROC_xferAck  : out std_logic;

    -- USER --
    ICLK             : in std_logic;
    IDQ              : in std_logic;
    ITRDY            : out std_logic;
    ITRI             : out std_logic;
    IGPH             : in std_logic;
    IGPV             : in std_logic;
    IGP0             : in std_logic;
    IGP1             : in std_logic;
    IPD              : in std_logic_vector(7 downto 0);
    HPD              : in std_logic_vector(7 downto 0)
    --VID_I2C_SCL      : inout std_logic;
    --VID_I2C_SDA      : inout std_logic    
  );
 
end entity opb_vidprocessor; --USER-- 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of opb_vidprocessor is --USER-- 

  component comparator
     port (
       clk : in std_logic;
       load  : in std_logic;
       current_lum   : in std_logic_vector(7 downto 0);
       output_lum : out std_logic_vector (7 downto 0);
       max_x : out std_logic_vector(8 downto 0);
       max_y : out std_logic_vector(8 downto 0)
       );
   end component;
   
  
-- component IOBUF_F_12
--  port (
--     O : out STD_ULOGIC;
--     IO : inout STD_ULOGIC;
--     I : in STD_ULOGIC;
--     T : in STD_ULOGIC);
-- end component;
 
  signal wdata : std_logic_vector(0 to 7); 
  signal rdata : std_logic_vector(0 to 7);
  signal rnw  : std_logic;
  signal cs, xfer : std_logic;
  signal q0,q1 : std_logic;
  signal i2c_din : std_logic;

  signal op_lum_l               : std_logic_vector(7 downto 0);
  signal x_l                    : std_logic_vector(8 downto 0);
  signal y_l                    : std_logic_vector(8 downto 0);
  signal op_lum_r               : std_logic_vector(7 downto 0);
  signal x_r                    : std_logic_vector(8 downto 0);
  signal y_r                    : std_logic_vector(8 downto 0);
   
  signal count_x                : std_logic_vector(9 downto 0);
  signal count_y                : std_logic_vector(9 downto 0);
  signal load_left              : std_logic;
  signal load_right             : std_logic;
  signal end_frame              : std_logic;
  signal load                   : std_logic_vector(1 downto 0);
  signal counter                : std_logic_vector(7 downto 0);
  signal counter1               : std_logic_vector(7 downto 0):= "00000011";
  signal counter2               : std_logic_vector(7 downto 0);
  signal counter3               : std_logic_vector(7 downto 0);
  signal counter4               : std_logic_vector(7 downto 0);
  signal ready_bit              : std_logic;
begin


-- sda_pad : IOBUF_F_12 port map (
--   I => wdata(0),
--   IO => VID_I2C_SDA,
--   O =>  i2c_din,
--   T => wdata(1)
-- );

-- scl_pad : IOBUF_F_12 port map (
--   I => wdata(2),
--   IO => VID_I2C_SCL,
--   O => open,
--   T => wdata(3)
-- );
  
-- Chip select, memory mapped. XIoOut8 for selecting the I2C controller
process (OPB_select, OPB_ABus)
begin

  if(OPB_select='1' and OPB_ABus(0 to 23)=C_BASEADDR(0 to 23)) then
    cs <= '1'; else
    cs <= '0';
  end if;  
end process;

-- Comparator Process
 process(ICLK)
  begin
   if ICLK'event and ICLK='1' then
    if IDQ ='1'  then
      end_frame <= '0';
      -- at end of line (639)
      if (count_x = "1001111111") then
        -- at end of frame (479)
        if (count_y = "0111011111") then
          load_right <= '1';
          load_left  <= '0';
          count_y  <= "0000000000";
          count_x  <= "0000000000";
          end_frame <= '1';
        else
          count_x  <= "0000000000";
          count_y <= count_y + 1;
          load_right <= '1';
          load_left  <= '0';
        end if;
      -- at left side (319)
      elsif (count_x < "0101000000") then
        load_left  <= '1';
        load_right <= '0';
        count_x <= count_x + 1;
     -- at right side
      else
       load_right <= '1';
       load_left  <= '0';
       count_x <= count_x + 1;
      end if;
    end if;
   end if;  
  end process;

-- I2C Bus SDA interconnection
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst='1') then
    wdata <= "11111111";
    rdata <= "00000000";
  elsif OPB_Clk'event and OPB_Clk = '1' then
    rnw <= OPB_RNW;
    if (q1 = '0' and q0 = '1' and rnw='0') then
       if OPB_ABus(29 to 31) = "111" then
         ready_bit <= OPB_DBus(31); 
       end if;
       wdata <= OPB_DBus(0 to 7);
    end if;
    if (q1='1') then
       rdata <= "00000000";
    elsif (q1='0' and q0='1' and rnw = '1') then
      if OPB_ABus(28 to 31) = "1111" then
        rdata <= "00000111";
      elsif OPB_ABus(29 to 31) = "000"  then
        rdata <= "00000011";
      elsif OPB_ABus(29 to 31) = "010" then
        rdata <= counter3;
      elsif OPB_ABus(29 to 31) = "011" then
        rdata <= counter4;
      elsif OPB_ABus(29 to 31) = "100" then
        rdata <= ready_bit & "0000000";
      else
        rdata <= "0000000";    
      end if;
      -- rdata <= "00000011"; --i2c_din & "0000000";
    end if;    
  end if;
end process;

process(OPB_Clk)
  begin
     if OPB_Clk' event and OPB_Clk = '1' then
       counter <= counter + 1;
       if counter = "11111111" then
         counter1 <= counter1 + 1;
         counter2 <= counter2 + 1;
         counter3 <= counter3 + 1;
         counter4 <= counter4 + 1;
         ready_bit <= '1';
       end if;
     end if;
end process;

--counter2 <= counter(7);  
  
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst = '1') then
    q0 <= '0';
    q1 <= '0';
  elsif OPB_Clk'event and OPB_Clk='1' then
    q1 <= not q1 and q0;
    q0 <= not q1 and not q0 and cs;
  end if;
end process;

xfer <= q1;

VIDPROC_xferAck <= xfer;

VIDPROC_DBus <= rdata & X"000000";

VIDPROC_errAck <= '0';

VIDPROC_retry <= '0';

VIDPROC_toutSup <= '0';

 comparator_left : comparator port map (
    clk         => ICLK,
    load        => load_left,
    current_lum => IPD,
    output_lum  => op_lum_l,
    max_x       => x_l,
    max_y       => y_l
    );

  comparator_right : comparator port map (
    clk         => ICLK,
    load        => load_right,
    current_lum => IPD,
    output_lum  => op_lum_r,
    max_x       => x_r,
    max_y       => y_r
    );

end architecture imp;
