library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity opb_vidprocessor is  -- USER --
  generic
  (
    C_OPB_AWIDTH       : integer          := 32;
    C_OPB_DWIDTH       : integer          := 32;
    C_BASEADDR         : std_logic_vector := X"FEFF0000";
    C_HIGHADDR         : std_logic_vector := X"FEFF00FF");
  
  port
  (
    --Required OPB bus ports, do not add to or delete
    OPB_ABus     : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE       : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk      : in  std_logic;
    OPB_DBus     : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW      : in  std_logic;
    OPB_Rst      : in  std_logic;
    OPB_select   : in  std_logic;
    OPB_seqAddr  : in  std_logic;
    VIDPROC_DBus     : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    VIDPROC_errAck   : out std_logic;
    VIDPROC_retry    : out std_logic;
    VIDPROC_toutSup  : out std_logic;
    VIDPROC_xferAck  : out std_logic;

    -- USER --
    ICLK             : in std_logic;
    IDQ              : in std_logic;
    ITRDY            : out std_logic;
    ITRI             : out std_logic;
    IGPH             : in std_logic;
    IGPV             : in std_logic;
    IGP0             : in std_logic;
    IGP1             : in std_logic;
    IPD              : in std_logic_vector(7 downto 0);
    HPD              : in std_logic_vector(7 downto 0)
    --VID_I2C_SCL      : inout std_logic;
    --VID_I2C_SDA      : inout std_logic    
  );
 
end entity opb_vidprocessor; --USER-- 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of opb_vidprocessor is --USER-- 

  component comparator
     port (
       clk : in std_logic;
       load  : in std_logic;
       current_lum   : in std_logic_vector(7 downto 0);
       output_lum : out std_logic_vector (7 downto 0);
       max_x : out std_logic_vector(8 downto 0);
       max_y : out std_logic_vector(8 downto 0)
       );
   end component;

-- added
   component IBUF
     port (
      I : in std_logic;
       O:  out std_logic
       );
   end component;
-- added
  
-- component IOBUF_F_12
--  port (
--     O : out STD_ULOGIC;
--     IO : inout STD_ULOGIC;
--     I : in STD_ULOGIC;
--     T : in STD_ULOGIC);
-- end component;
 
  signal wdata : std_logic_vector(0 to 7); 
  signal rdata : std_logic_vector(0 to 7);
  signal rnw  : std_logic;
  signal cs, xfer : std_logic;
  signal q0,q1 : std_logic;
  signal i2c_din : std_logic;

  signal op_lum_l               : std_logic_vector(7 downto 0);
  signal x_l                    : std_logic_vector(8 downto 0);
  signal y_l                    : std_logic_vector(8 downto 0);
  signal op_lum_r               : std_logic_vector(7 downto 0);
  signal x_r                    : std_logic_vector(8 downto 0);
  signal y_r                    : std_logic_vector(8 downto 0);
   
  signal count_x                : std_logic_vector(9 downto 0);
  signal count_y                : std_logic_vector(9 downto 0);
  signal load_left              : std_logic;
  signal load_right             : std_logic;
  signal end_frame              : std_logic;
  signal load                   : std_logic_vector(1 downto 0);
  signal counter                : std_logic_vector(7 downto 0);
  signal counter1               : std_logic_vector(7 downto 0);
  signal counter2               : std_logic_vector(7 downto 0);
  signal counter3               : std_logic_vector(7 downto 0);
  signal counter4               : std_logic_vector(7 downto 0);
  signal ready_bit              : std_logic:='1';
  signal buf_IPD                : std_logic_vector(7 downto 0);
  signal buf_IDQ                : std_logic;
  signal buf_ICLK               : std_logic;
  
begin


-- sda_pad : IOBUF_F_12 port map (
--   I => wdata(0),
--   IO => VID_I2C_SDA,
--   O =>  i2c_din,
--   T => wdata(1)
-- );

-- scl_pad : IOBUF_F_12 port map (
--   I => wdata(2),
--   IO => VID_I2C_SCL,
--   O => open,
--   T => wdata(3)
-- );
  
-- Chip select, memory mapped. XIoOut8 for selecting the I2C controller
process (OPB_select, OPB_ABus)
begin

  if(OPB_select='1' and OPB_ABus(0 to 23)=C_BASEADDR(0 to 23)) then
    cs <= '1'; else
    cs <= '0';
  end if;  
end process;

-- Comparator Process
 process(buf_ICLK)
  begin
   if buf_ICLK'event and buf_ICLK='1' then
    if buf_IDQ ='1'  then
      end_frame <= '0';
      -- at end of line (639)
      if (count_x = "1001111111") then
        -- at end of frame (479)
        if (count_y = "0111011111") then
          load_right <= '1';
          load_left  <= '0';
          count_y  <= "0000000000";
          count_x  <= "0000000000";
          end_frame <= '1';
        else
          count_x  <= "0000000000";
          count_y <= count_y + 1;
          load_right <= '1';
          load_left  <= '0';
        end if;
      -- at left side (319)
      elsif (count_x < "0101000000") then
        load_left  <= '1';
        load_right <= '0';
        count_x <= count_x + 1;
     -- at right side
      else
       load_right <= '1';
       load_left  <= '0';
       count_x <= count_x + 1;
      end if;
    end if;
   end if;  
  end process;

-- I2C Bus SDA interconnection
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst='1') then
    wdata <= "11111111";
    rdata <= "00000000";
  elsif OPB_Clk'event and OPB_Clk = '1' then
    rnw <= OPB_RNW;
    if (q1 = '0' and q0 = '1' and rnw='0') then
       wdata <= OPB_DBus(0 to 7);
    end if;
    if (q1='1') then
       rdata <= "00000000";
    elsif (q1='0' and q0='1' and rnw = '1') then
      if OPB_ABus(24 to 31) = X"00" then
        rdata <= wdata;
      elsif OPB_ABus(24 to 31) = X"04"  then
        rdata <= buf_IPD;
      elsif OPB_ABus(24 to 31) = X"08" then
        rdata <= "00000001";
      elsif OPB_ABus(24 to 31) = X"0C" then
        rdata <= "00000001";--ready_bit & "0000000";
      elsif OPB_ABus(24 to 31) = X"10" then
        rdata <= "00000001";
      else
        rdata <= "00000000";    
      end if;
      -- rdata <= "00000011"; --i2c_din & "0000000";
    end if;    
  end if;
end process;

-- process(OPB_Clk)
--   begin
--      if OPB_Clk' event and OPB_Clk = '1' then
--        counter <= counter + 1;
--        if counter = "11111111" then
--          counter1 <= counter1 + 1;
--          counter2 <= counter2 + 1;
--          counter3 <= counter3 + 1;
--          counter4 <= counter4 + 1;
--          wdata    <= "00000000";
--        end if;
--      end if;
-- end process;

--counter2 <= counter(7);  
  
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst = '1') then
    q0 <= '0';
    q1 <= '0';
  elsif OPB_Clk'event and OPB_Clk='1' then
    q1 <= not q1 and q0;
    q0 <= not q1 and not q0 and cs;
  end if;
end process;

xfer <= q1;

VIDPROC_xferAck <= xfer;

VIDPROC_DBus <= rdata & X"000000";

VIDPROC_errAck <= '0';

VIDPROC_retry <= '0';

VIDPROC_toutSup <= '0';

 comparator_left : comparator port map (
    clk         => buf_ICLK,
    load        => load_left,
    current_lum => buf_IPD,
    output_lum  => op_lum_l,
    max_x       => x_l,
    max_y       => y_l
    );

  comparator_right : comparator port map (
    clk         => buf_ICLK,
    load        => load_right,
    current_lum => buf_IPD,
    output_lum  => op_lum_r,
    max_x       => x_r,
    max_y       => y_r
    );
 
    bufipd0 : IBUF port map (
     I => IPD(0),
     O => buf_IPD(0)
     );

    bufipd1 : IBUF port map (
     I => IPD(1),
     O => buf_IPD(1)
     );
    bufipd2 : IBUF port map (
     I => IPD(2),
     O => buf_IPD(2)
     );
   bufipd3 : IBUF port map (
     I => IPD(3),
     O => buf_IPD(3)
     );
    bufipd4 : IBUF port map (
     I => IPD(4),
     O => buf_IPD(4)
     );
    bufipd5 : IBUF port map (
     I => IPD(5),
     O => buf_IPD(5)
     );
    bufipd6 : IBUF port map (
     I => IPD(6),
     O => buf_IPD(6)
     );
    bufipd7 : IBUF port map (
     I => IPD(7),
     O => buf_IPD(7)
     );
    bufidq : IBUF port map (
     I => IDQ,
     O => buf_IDQ
     );
    buficlk : IBUF port map (
     I => ICLK,
     O => buf_ICLK
     );
 
  
end architecture imp;
